"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TemplateUrlsResolver = exports.getTemplateUrls = exports.hasTemplateUrl = exports.StyleUrlsResolver = exports.hasStyleUrls = void 0;
const path_1 = require("path");
const ts_morph_1 = require("ts-morph");
const vite_1 = require("vite");
const styleUrlsRE = /styleUrls\s*:\s*\[([^\[]*?)\]|styleUrl:\s*["'](.*?)["']/;
function hasStyleUrls(code) {
    return styleUrlsRE.test(code);
}
exports.hasStyleUrls = hasStyleUrls;
const EMPTY_ARRAY = [];
class StyleUrlsResolver {
    // These resolvers may be called multiple times during the same
    // compilation for the same files. Caching is required because these
    // resolvers use synchronous system calls to the filesystem, which can
    // degrade performance when running compilations for multiple files.
    styleUrlsCache = new Map();
    resolve(code, id) {
        const styleUrlsExecArray = styleUrlsRE.exec(code);
        if (styleUrlsExecArray === null) {
            return EMPTY_ARRAY;
        }
        // Given the code is the following:
        // @Component({
        //   styleUrls: [
        //     './app.component.scss'
        //   ]
        // })
        // The `matchedStyleUrls` would result in: `styleUrls: [\n    './app.component.scss'\n  ]`.
        const [matchedStyleUrls] = styleUrlsExecArray;
        const entry = this.styleUrlsCache.get(id);
        // We're using `matchedStyleUrls` as a key because the code may be changing continuously,
        // resulting in the resolver being called multiple times. While the code changes, the
        // `styleUrls` may remain constant, which means we should always return the previously
        // resolved style URLs.
        if (entry?.matchedStyleUrls === matchedStyleUrls) {
            return entry.styleUrls;
        }
        // The `styleUrls` property is an array, which means we may have a list of
        // CSS files provided there. Let `matchedStyleUrls` be equal to the following:
        // "styleUrls: [\n    './app.component.scss',\n    '../global.scss'\n  ]"
        const styleUrlPaths = matchedStyleUrls
            .replace(/(styleUrls|\:|\s|\[|\]|"|')/g, '')
            .replace(/(styleUrl|:\s*["'](.*?)["'])/g, '')
            // The above replace will result in the following:
            // "./app.component.scss,../global.scss"
            .split(',');
        const styleUrls = styleUrlPaths.map((styleUrlPath) => {
            return `${styleUrlPath}|${(0, vite_1.normalizePath)((0, path_1.resolve)((0, path_1.dirname)(id), styleUrlPath))}`;
        });
        this.styleUrlsCache.set(matchedStyleUrls, { styleUrls, matchedStyleUrls });
        return styleUrls;
    }
}
exports.StyleUrlsResolver = StyleUrlsResolver;
function hasTemplateUrl(code) {
    return code.includes('templateUrl:');
}
exports.hasTemplateUrl = hasTemplateUrl;
function getTemplateUrls(code) {
    const project = new ts_morph_1.Project({ useInMemoryFileSystem: true });
    const sourceFile = project.createSourceFile('cmp.ts', code);
    return sourceFile
        .getDescendantsOfKind(ts_morph_1.SyntaxKind.PropertyAssignment)
        .filter((property) => property.getName() === 'templateUrl')
        .map((property) => property.getInitializer()?.getText().replace(/['"]/g, ''))
        .filter((url) => url !== undefined);
}
exports.getTemplateUrls = getTemplateUrls;
class TemplateUrlsResolver {
    templateUrlsCache = new Map();
    resolve(code, id) {
        const entry = this.templateUrlsCache.get(id);
        if (entry?.code === code) {
            return entry.templateUrlPaths;
        }
        const templateUrlPaths = getTemplateUrls(code).map((url) => `${url}|${(0, vite_1.normalizePath)((0, path_1.resolve)((0, path_1.dirname)(id), url).replace(/\\/g, '/'))}`);
        this.templateUrlsCache.set(id, { code, templateUrlPaths });
        return templateUrlPaths;
    }
}
exports.TemplateUrlsResolver = TemplateUrlsResolver;
//# sourceMappingURL=component-resolvers.js.map